/***************************************************************************
                          kpogre.h  -  description
                             -------------------
    begin                : Fre Sep  7 16:06:25 CEST 2001
    copyright            : (C) 2002 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPOGRE_H
#define KPOGRE_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>

#include "DbObjects/kpgtreeitem.h"
#include "kpogreview.h"

// forward declaration of the KPoGre classes
class KPoGreDoc;
class KPGConfiguration;
class KPGBookmarksButton;
class KPGSearchDialog;
class KPoGreComboAction;

/**
  * The base class for KPoGre application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KPoGreView creates your center view, which is connected
  * to the window's Doc object.
  * KPoGreApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */
class KPoGreApp : public KMainWindow
{
  Q_OBJECT

  friend class KPoGreView;

  public:
    /** construtor of KPoGreApp, calls all init functions to create the application.
     */
    KPoGreApp(QWidget* parent=0, const char* name=0);
    ~KPoGreApp();
    
    /** opens a file specified by commandline option
     */
    void openDocumentFile(const KURL& url=0);
    
    /** returns a pointer to the current document connected to the KTMainWindow instance and is used by
     * the View class to access the document object's methods
     */	
    KPoGreDoc *getDocument() const;

    static KPGConfiguration * configuration();
    
    DCOPClient *dcopClient() { return kapp->dcopClient(); }

  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */ 	
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initializes the KActions of the application */
		
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
		
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    void initView();
    /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** saves the window properties for each open window during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    //virtual void saveProperties(KConfig *_cfg);
    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    //virtual void readProperties(KConfig *_cfg);

  public slots:
    
    //--- File main menu actions
    void slotFileNewWindow();
    void slotFileNew();
    void slotFileOpen();
    void slotFileOpenRecent(const KURL& url);
    void slotFileSave();
    void slotFileSaveAs();
    void slotExportResult();
    void slotFileClose();
    void slotFilePrint();
    void slotFileQuit();
    
		//--- Edit main menu actions
		void slotEditUndo();
	  void slotEditRedo();
    void slotEditCut();
		void slotEditCopy();
		void slotEditPaste();
    void slotEditFind();
		void slotEditFindNext();
    void slotEditDeselect();
    void slotEditClearInputArea();
    
    //--- View main menu actions
    void slotViewNodeUp() { view->viewNodeUp(); }
    void slotViewNodeBack() { view->viewNodeBack(); }
    void slotViewNodeForward() { view->viewNodeForward(); }
		
    //--- Settings main menu actions
		void slotViewToolBar();
    void slotViewStatusBar();
    void slotConfigureKeys();
    void slotConfigToolbars();
    void slotConfigure();
    
		//--- Database main menu actions
    void slotDatabaseConnect();
		void slotDatabaseDisconnect();
    void slotDatabaseRunQuery();
    void slotDatabaseStopQuery();
    void slotDatabaseFetchNext();
    void slotDatabaseFetchAll();
		void slotDatabaseWizard();
		
    //--- Other slots
    void slotStatusMsg(const QString &); // changes the statusbar contents for the standard label
    
			
		/** Called, when document is modified. Enable Save button. */
		void slotDocumentModified();
	
		// Called, when user change selection in tree view
		void slotSelectionChanged( const KPGTreeItem::eNodeType, KPGTreeItem *, unsigned int, unsigned int);
	
		// Enables/disables action Database -> Run SQL
		void slotEnableRunQueryAction(bool);
		
    // Enables/disables action Database -> Stop SQL
    void slotEnableStopQueryAction(bool);
    
    // Enables/disables actiond Database -> Fetch Next, Fetch All
    void slotEnableDbFetchActions(bool, bool);
        
		// Enables/disables File Open, Save and Export action
		void slotEnableFileActions(bool, bool, bool);
			
		// Enables/disables Edit actions
		void slotEnableEditActions(bool, bool, bool, bool, bool, bool);
		
		// Called when user press Bookmarks Button
		void slotBookmarksButtonPressed();
		
		// Called when user choose bookmark
		void slotOpenUrl( const QString& );
		
		// Display popup menu
		void slotContextMenuRequested(const QString &, const QPoint &);
			
		// Refresh selected item
		void slotRefresh();
		
		void slotPathSelected( const QString & szPath );
			
		void slotPathClear();
		
    //-------------------------------------------------------------------------
    //
    // Object menu actions fired from left tree view, when user right-click on object 
    //
    //-------------------------------------------------------------------------
		
		//--- Database related menu actions
		void slotCreateDatabase();
		void slotDropDatabase();
    void slotAnalyseDatabase();
    void slotReindexDatabase();
    void slotSetOwnerDatabase();
    void slotGrantPrivilegesDatabase();
    void slotRevokePrivilegesDatabase();
    void slotDumpDatabase();
    void slotRestoreDatabase();
    void slotDisconnectDatabase();
		
    //--- Schema related menu actions
		void slotCreateSchema();
	  void slotDropSchema();
		void slotModifySchema();
    void slotSetOwnerSchema();
    void slotGrantPrivilegesSchema();
    void slotRevokePrivilegesSchema();
		
    //--- Table related menu actions
    void slotCreateTable();
	  void slotDropTable();
    void slotRenameTable();
    void slotAnalyseTable();
    void slotVacuumTable();
    void slotReindexTable();
    void slotSetClusterOnTable();
    void slotSetWithoutClusterTable();
    void slotClusterTable();
    void slotCopyFromTable();
    void slotCopyToTable();
    void slotSetOwnerTable();
    void slotGrantPrivilegesTable();
    void slotRevokePrivilegesTable();
    void slotSetTablespaceTable();
    void slotSetWithoutOIDsTable();
    
		//--- View related menu actions
    void slotCreateView();
		void slotDropView();
		void slotModifyView();
    void slotGrantPrivilegesView();
    void slotRevokePrivilegesView();
		    
    //--- Tablespaces related menu actions
    void slotCreateTablespace();
    void slotDropTablespace();
    void slotRenameTablespace();
    void slotSetOwnerTablespace();
    void slotGrantPrivilegesTablespace();
    void slotRevokePrivilegesTablespace();
    
    //--- User related menu actions
    void slotCreateUser();
		void slotDropUser();
		void slotModifyUser();
				
		//--- Group related menu actions
		void slotCreateGroup();
		void slotDropGroup();
		void slotModifyGroup();
        
    //--- Language related menu actions
    void slotCreateLanguage();
    void slotDropLanguage();
    void slotModifyLanguage();
    void slotGrantPrivilegesLanguage();
    void slotRevokePrivilegesLanguage();
    
    //--- Domain related menu actions 
    void slotCreateDomain();
    void slotDropDomain();
    void slotSetDomainNull();
    void slotSetDomainDefault();
    void slotDropDomainDefault();
    void slotSetOwnerDomain();
    
    //--- Domain constraint related menu actions
    void slotCreateDomainConstraint();
    void slotDropDomainConstraint();
    
    //--- Function related menu actions
    void slotCreateFunction();
    void slotDropFunction();
    void slotRenameFunction();
    void slotModifyFunction();
    void slotGrantPrivilegesFunction();
    void slotRevokePrivilegesFunction();
    
    //--- Sequence related menu actions
    void slotCreateSequence();
    void slotDropSequence();
    void slotModifySequence();
    void slotGrantPrivilegesSequence();
    void slotRevokePrivilegesSequence();
        
    //--- Column related menu actions
    void slotCreateColumn();
    void slotDropColumn();
    void slotRenameColumn();
    void slotSetColumnNull();
    void slotSetColumnDefault();
    void slotDropColumnDefault();
    void slotChangeColumnType();
    void slotSetColumnStorage();
    void slotSetColumnStatistisc();
    
    //--- Constraint related menu actions
    void slotCreateConstraint();
    void slotDropConstraint();
    
    //--- Index related menu actions
    void slotCreateIndex();
    void slotDropIndex();
    void slotRenameIndex();
    void slotReindexIndex();
    void slotSetOwnerIndex();
    void slotSetTablespaceIndex();
        
    //--- Rule related menu actions
    void slotCreateRule();
    void slotDropRule();
    
    //--- Trigger related menu actions
    void slotCreateTrigger();
    void slotDropTrigger();
    void slotRenameTrigger();
    
    //--- Type related menu actions
    void slotCreateBaseType();
    void slotCreateCompositeType();
    void slotDropType();
    void slotSetOwnerType();
    
private:
    /** the configuration object of the application */
    KConfig *config;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    KPoGreView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    KPoGreDoc *doc;

    // KAction pointers to enable/disable actions
    
    KAction *m_pActFileExportResult;
    KAction* m_pActFileNewWindow;
    KAction* m_pActFileNew;
    KAction* m_pActFileOpen;
    KRecentFilesAction* m_pActFileOpenRecent;
    KAction* m_pActFileSave;
    KAction* m_pActFileSaveAs;
    KAction* m_pActFileClose;
    KAction* m_pActFilePrint;
    KAction* m_pActFileQuit;
    
		KAction* m_pActEditUndo;
		KAction* m_pActEditRedo;
		KAction* m_pActEditCut;
    KAction* m_pActEditCopy;
    KAction* m_pActEditPaste;
		KAction* m_pActEditFind;
		KAction* m_pActEditFindNext;
		KAction* m_pActEditClearInputArea;
		KAction* m_pActEditDeselect;
		
		
		KAction* m_pActViewNodeUp;
		KAction* m_pActViewNodeBack;
		KAction* m_pActViewNodeForward;
    KToggleAction* m_pActViewToolBar;
    KToggleAction* m_pActViewStatusBar;
    
    KAction* m_pActServerRegister;
    KAction* m_pActDatabaseConnect;
    KAction* m_pActDatabaseDisconnect;
    KAction* m_pActDatabaseRunQuery;
    KAction* m_pActDatabaseStopQuery;
    KAction* m_pActDatabaseFetchNext;
    KAction* m_pActDatabaseFetchAll;
		KAction* m_pActDatabaseWizard;
		
		KAction* m_pActFolderRefresh;

		KPoGreComboAction* m_pActPathCombo;
		
    /** our configuration - the same for all parts */
		static KPGConfiguration* s_pKPGConfig;

    KPGBookmarksButton *m_pBookmarksButton;
    
		/** the search dialog, used for find action */
		KPGSearchDialog* m_pDlgSearch; 
};
 
#endif // KPOGRE_H
